import pymongo

from DuyanCommon.CommonUtls import EntityPage, Entity
from Error import PageError, SizeError, OverSizeLimitError


class BasicDBObject(dict):
    EQUALS = "$eq"
    NOT_EQUALS = "$ne"
    GREATER_THEN = "$gt"
    GREATER_THEN_EQUALS = "$gte"
    LESS_THEN = "$lt"
    LESS_THEN_EQUALS = "$lte"
    REGEX = "$regex"
    IN = "$in"
    NOT_IN = "$nin"
    MATCH = "$match"
    GROUP = "$group"
    SUM = "$sum"
    EXISTS = "$exists"
    COND = "$cond"

    @staticmethod
    def field_value_key(field_name):
        """
        :param field_name:
        :return:
        """
        return f"${field_name}"

    def cond(self, conditions, hit_value, else_value):
        return self.put(self.COND, [conditions, hit_value, else_value])

    def eq(self, key: str, value):
        """
            等于
        """
        return self.put(key, {self.EQUALS: value})

    def ne(self, key: str, value):
        """
            不等于
        """
        return self.put(key, {self.NOT_EQUALS: value})

    def gt(self, key, value):
        """
            大于
        """
        return self.put(key, {self.GREATER_THEN: value})

    def gte(self, key, value):
        """
            大于等于
        """
        return self.put(key, {self.GREATER_THEN_EQUALS: value})

    def lt(self, key, value):
        """
            小于
        """
        return self.put(key, {self.LESS_THEN: value})

    def lte(self, key, value):
        """
            小于等于
        """
        return self.put(key, {self.LESS_THEN_EQUALS: value})

    def between(self, key, min_value, max_value):
        return self.put(key, {self.GREATER_THEN_EQUALS: min_value, self.LESS_THEN_EQUALS: max_value})

    def range(self, key, min_value, max_value, include_min=False, include_max=False):
        """
            范围查询 ， 默认前后不包括
        """
        return self.put(key,
                        {include_min and self.GREATER_THEN_EQUALS or self.GREATER_THEN: min_value,
                         include_max and self.LESS_THEN_EQUALS or self.LESS_THEN: max_value})

    def like(self, key, regex_value):
        """
            模糊匹配
        :param key:
        :param regex_value:
        :return:
        """
        return self.put(key, {self.REGEX: regex_value})

    def in_(self, key, value: list):
        return self.put(key, {self.IN: value})

    def not_in(self, key, value: list):
        return self.put(key, {self.NOT_IN: value})

    def sum(self, name, field_name):
        return self.put(name, {self.SUM: field_name})

    def exists(self, key):
        return self.put(key, {self.EXISTS: True})

    def not_exists(self, key):
        return self.put(key, {self.EXISTS: False})

    def match(self, query: dict):
        return self.put(self.MATCH, query)

    def group(self, aggregate: dict):
        return self.put(self.GROUP, aggregate)

    def put(self, key, value):
        if isinstance(key, bytes):
            key = str(key, encoding='utf-8')
        key = str(key)
        if key in self.keys():
            raise Exception(f"字段{key}条件已存在")
        self[key] = value
        return self


class MongoDBClient(object):

    def __init__(self, host: str, db_name: str, table_name: str, limit: int = 0):
        """
            mongo 客户端
        :param host:
        :param db_name:
        :param table_name:
        :param limit: 查询最大数量，防止内存不足问题 ，不设置不会对查询结果进行限制
        """
        self.host = host
        self.db_name = db_name
        self.table_name = table_name
        self.client = pymongo.MongoClient(self.host)
        self.db = self.client.get_database(self.db_name)
        self.table = self.db.get_collection(self.table_name)
        self.limit = limit if limit is not None and limit > 0 else None

    def switch_table(self, table_name):
        self.table_name = table_name
        self.table = self.db.get_collection(self.table_name)

    def __find__(self, query: BasicDBObject, order_by: list = None, page: int = None, size: int = None):
        """
            基础查询 find 的方法都给予此
        :param query:  查询sql
        :param order_by: 排序方式 ，列表例如 ：[("field_name" , pymongo.DESCENDING)]
        :param page: 分页
        :param size: 每页展示条数
        :return:
        """
        cur = self.table.find(query)

        if order_by is not None:
            cur = cur.sort(order_by)
        if page is not None and size is not None and size > 0:
            cur = cur.skip(page * size).limit(size)
        elif size is not None and size > 0:
            cur = cur.limit(size)
        return cur

    def find_one(self, query: BasicDBObject, order_by: list = None) -> Entity:
        """
            查找一条
        :param query:
        :return:
        """
        entity_obj = self.__find__(query, order_by=order_by, size=1)
        return Entity(entity_obj[0]) if entity_obj is not None and len(entity_obj) > 0 else None

    def count(self, query: BasicDBObject) -> int:
        """
            计数
        :param query:
        :return:
        """
        return self.table.count_documents(query)

    def find(self, query: BasicDBObject, order_by=None, ) -> list[Entity]:
        """
            查询 不分页
        :param query:
        :param order_by:
        :return:
        """
        data = self.__find__(query, order_by=order_by,
                             size=self.limit if self.limit is not None and self.limit > 0 else None)
        return Entity.build_list(data)

    def find_page(self, query: BasicDBObject, page: int, size: int, order_by=None) -> EntityPage:
        """
            分页查询
        :param query:
        :param page:
        :param size:
        :param order_by:
        :return:
        """
        if page is None or page < 0:
            raise PageError
        if size is None or size <= 0:
            raise SizeError
        if self.limit is not None and 0 < self.limit < size:
            raise OverSizeLimitError
        data = self.__find__(query, order_by, page, size)
        total = self.count(query)
        return EntityPage.build_obj(data, page, size, total)

    def group(self, match: BasicDBObject, group: BasicDBObject) -> list[Entity]:
        """
            聚合 这里能完成简单的聚合并提供对象
        :param match:
        :param group:
        :return:
        """
        if match.get(BasicDBObject.MATCH) is None:
            match_obj = {BasicDBObject.MATCH: match}
        else:
            match_obj = match
        if group.get(BasicDBObject.GROUP) is None:
            group_obj = {BasicDBObject.GROUP: group}
        else:
            group_obj = group
        data = self.table.aggregate([match_obj, group_obj])
        entities = list()
        for item in data:
            entity = Entity()
            for key, value in item.items():
                if key == "_id":
                    for k, v in value.items():
                        entity.__setattr__(str(k), v)
                else:
                    entity.__setattr__(str(key), value)
            entities.append(entity)
        return entities






import re

import pymysql
import loguru

from DuyanCommon.CommonUtls import Entity, EntityPage

logger = loguru.logger

try:
    """支持DBUtils>=2.0以上版本"""
    from dbutils.pooled_db import PooledDB

    DB_VERSION = PooledDB.version
    logger.info("当前DBUtils版本-{}", DB_VERSION)
except Exception as e:
    """用于兼容DBUtils<2.0以下版本"""

    from DBUtils.PooledDB import PooledDB

    DB_VERSION = PooledDB.version
    logger.warning("当前DBUtil[{}]版本过低...请适当升级版本至最新版", DB_VERSION)

from Error import VersionLowError, PageError, SizeError, ParamError, SQLFormatError, SQLNotInsertError, \
    SQLNotDeleteError, SQLNotUpdateError, SQLNotSelectError


class MySQLDBClient(object):

    def __init__(self, host, user_name, password, db_name, port: int = 3306, mincached=1, maxcached=2,
                 maxshared=0, maxconnections=5, blocking=False,
                 maxusage=50, setsession=None, reset=True,
                 failures=None, ping=1,
                 *args, **kwargs):
        self.pool_db_version = PooledDB.version
        self.pool = PooledDB(pymysql,
                             host=host or "",
                             user=user_name or "",
                             passwd=password or "",
                             db=db_name or "",
                             port=port or 3306,
                             mincached=mincached,
                             maxcached=maxcached,
                             maxshared=maxshared,
                             maxconnections=maxconnections,
                             blocking=blocking,
                             maxusage=maxusage,
                             setsession=setsession,
                             reset=reset,
                             failures=failures,
                             ping=ping)

    def select_one(self, sql: str, *args, **kwargs) -> Entity:
        if not sql.rstrip().lower().startswith("select"):
            raise SQLNotSelectError(sql)
        conn = None
        cursor = None
        try:
            conn = self.pool.connection()
            cursor = conn.cursor(pymysql.cursors.DictCursor)
            self.__execute__(cursor, sql, *args, **kwargs)
            result = cursor.fetchone()
            logger.debug(f"<== Total:{result and 1 or 0}")
            return Entity(data=result) if result is not None else None
        except Exception as e:
            loguru.logger.exception(e)
        finally:
            if cursor is not None:
                cursor.close()
            if conn is not None:
                conn.close()

    def select_page(self, sql: str, page: int, size: int, *args, **kwargs) -> EntityPage:
        """
            分页查询
        :param sql: 基础sql
        :param page: 页码
        :param size: 一页展示条数
        :param args: sql 变量
        :param kwargs: sql 变量
        :return:
        """
        if not sql.rstrip().lower().startswith("select"):
            raise SQLNotSelectError(sql)
        if page is None or page < 0:
            raise PageError
        if size is None or size <= 0:
            raise SizeError
        if "group by " in sql.lower():
            raise SQLFormatError(sql)
        data = self.select(sql + f" limit {page * size} , {size}", *args, **kwargs)
        # 这里去统计组装 count 的语句
        r = re.search(r"([\s\n]from[\s\n]).*", sql.lower())
        if r is None:
            raise SQLFormatError(sql)
        # 找到 from 后面的语句后 组装 count
        _sql = "SELECT COUNT(1) as c " + sql[-len(r.group()):]
        total = self.count(_sql, *args, **kwargs)
        return EntityPage.build_obj(data, page, size, total)

    def count(self, sql: str, *args, **kwargs) -> int:
        res_obj = self.select_one(sql, *args, **kwargs)
        return res_obj.c if res_obj is not None else 0

    def select(self, sql: str, *args, **kwargs) -> list[Entity]:
        if not sql.rstrip().lower().startswith("select"):
            raise SQLNotSelectError(sql)
        conn = None
        cursor = None
        try:
            conn = self.pool.connection()
            cursor = conn.cursor(pymysql.cursors.DictCursor)
            self.__execute__(cursor, sql, *args, **kwargs)
            result = cursor.fetchall()
            logger.debug(f"<== Total:{result and len(result) or 0}")
            return Entity.build_list(result)
        except Exception as e:
            logger.exception(e)
        finally:
            if cursor is not None:
                cursor.close()
            if conn is not None:
                conn.close()

    def insert(self, sql: str, *args, **kwargs) -> int:
        """执行插入"""
        if not sql.rstrip().lower().startswith("insert"):
            raise SQLNotInsertError(sql)
        return self.execute(sql, *args, **kwargs)

    def delete(self, sql, *args, **kwargs) -> int:
        if not sql.rstrip().lower().startswith("delete"):
            raise SQLNotDeleteError(sql)
        return self.execute(sql, *args, **kwargs)

    def update(self, sql, *args, **kwargs) -> int:
        if not sql.rstrip().lower().startswith("update"):
            raise SQLNotUpdateError(sql)
        return self.execute(sql, *args, **kwargs)

    def execute(self, sql, *args, **kwargs) -> int:
        """
            执行器 主要处理 update delete insert 的语句
        :param sql:
        :param args:
        :param kwargs:
        :return:
        """
        conn = None
        cursor = None
        try:
            conn = self.pool.connection()
            cursor = conn.cursor(pymysql.cursors.DictCursor)
            self.__execute__(cursor, sql, *args, **kwargs)
            result = cursor.rowcount
            conn.commit()
            logger.debug(f"<== Total:{result or 0}")
            return result or 0
        except Exception as e:
            logger.exception(e)
        finally:
            if cursor is not None:
                cursor.close()
            if conn is not None:
                conn.close()

    def insert_many(self, sql, args=None) -> int:
        """
            批量插入
        :param sql: sql 格式
        :param args:
        :return:
        """
        if not sql.rstrip().lower().startswith("select"):
            raise SQLNotInsertError(sql)
        conn = None
        cursor = None
        try:
            conn = self.pool.connection()
            cursor = conn.cursor(pymysql.cursors.DictCursor)
            rowcount = self.__executemany__(cursor, sql, args)
            conn.commit()
            return rowcount
        except Exception as e:
            loguru.logger.exception(e)
        finally:
            if cursor is not None:
                cursor.close()
            if conn is not None:
                conn.close()

    @staticmethod
    def __execute__(cursor, sql, *args, **kwargs):
        """
            底层调用sql的方法
        :param cursor: 游标
        :param sql:  执行sql
        :param args:  参数
        :param kwargs: 参数
        :return:
        """
        _sql_format = sql.format(*args, **kwargs)
        logger.debug(f"==> Preparing:{sql}")
        if args is not None and len(args):
            logger.debug(f"==> Parameters:{args}")
        elif kwargs is not None and len(kwargs):
            logger.debug(f"==> Parameters:{kwargs}")
        cursor.execute(_sql_format)

    @staticmethod
    def __executemany__(cursor, sql, args=None) -> int:
        """
            底层调用 批量插入的sql方法
        :param cursor:  游标
        :param sql: insert 语句
        :param args: 数据
        :return:
        """
        logger.debug(f"==> Preparing:{sql}")
        if args is not None and len(args):
            logger.debug(f"==> Parameters:{args}")
        rowcount = cursor.executemany(sql, args=args)
        logger.debug(f"<== Total:{rowcount}")
        return rowcount


if __name__ == "__main__":
    client = MySQLDBClient(host="127.0.0.7", password="admin123", user_name="root", db_name="videoxt")
    print(client.select_page("select * from blogger where id >{0}", 4, 10, 100000))



# -*- coding: UTF-8 -*-
import json
import random
import re
import time
from datetime import datetime, date, timedelta
from typing import Tuple
from DuyanCommon.Error import TimeNoFormatError, ConversionTypeError, NullPointError, TimeConversionTypeError, ParamError


class Entity(object):
    def __init__(self, data: dict = None):
        if data is None:
            return
        for key in data:
            value = data.get(key)
            self.__setattr__(str(key), value)

    def __str__(self):
        var = dict()
        for key in self.__dir__():
            if key.startswith("__") or "build_list" == key:
                continue
            var[key] = self.__getattribute__(key)
        return str(var)

    @staticmethod
    def build_list(data_list: list[dict]):
        res = list()
        if data_list is None or len(data_list) == 0:
            return res
        for item in data_list:
            if isinstance(item, Entity):
                res.append(item)
            else:
                res.append(Entity(item))
        return res


class EntityPage(object):
    def __init__(self):
        self.page = None
        self.size = None
        self.total = None
        self.page_size = None
        self.data = None

    def __str__(self):
        var = dict()
        for key in self.__dir__():
            if key.startswith("__") or "build_obj" == key :
                continue
            if "data" == key:
                data = self.__getattribute__(key)
                var[key] = [str(i) for i in data]
            else:
                var[key] = self.__getattribute__(key)
        return str(var)

    @staticmethod
    def build_obj(data, page: int = None, size: int = None, total: int = None):
        entity = EntityPage()
        entity.data = Entity.build_list(data)
        entity.page = page
        entity.size = size
        entity.total = total
        entity.page_size = entity.data and len(entity.data) or 0
        return entity


class Utils(object):
    """
        通用工具
    """
    NUMERICAL_REGX = r"^\d+$"  # 数字类型正则
    EMAIL_REGX = r"^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$"
    MOBILE_REGX = r"^(\+86){0,1}0{0,1}1[3-9]\d{9}$"

    @staticmethod
    def get_random_int(start: int = 0, end: int = 9) -> int:
        """
            获取指定范围的随机数
        :param start:
        :param end:
        :return:
        """
        return random.randint(start, end)

    @staticmethod
    def get_random_len(_len: int = 1) -> str:
        if _len <= 0:
            raise ParamError
        x = pow(10, _len) - 1
        var = str(Utils.get_random_int(0, x))
        while len(var) < _len:
            var = "0" + var
        return var

    @staticmethod
    def is_empty(var):
        """
            判空
        :param var:
        :return:
        """
        if var is None or len(str(var)) == 0:
            return True
        if isinstance(var, str):
            return len(var.strip()) == 0
        if isinstance(var, list) or isinstance(var, dict) or isinstance(var, Tuple):
            return len(var) == 0
        return False

    @staticmethod
    def is_numerical(var: str):
        """
            判断是否是数字
        :param var:
        :return:
        """
        if Utils.is_empty(var):
            return False
        r = re.search(Utils.NUMERICAL_REGX, var)
        return r is not None

    @staticmethod
    def get_numerical(var: str):
        """
            拿数字
        :param var:
        :return:
        """
        if var is None:
            raise NullPointError()
        if not Utils.is_numerical(var):
            raise ConversionTypeError(f"{var}不是整型")
        return int(var)

    @staticmethod
    def is_email(var: str):
        """
            判断是否是email
        :param var:
        :return:
        """
        if Utils.is_empty(var):
            return False
        r = re.search(Utils.EMAIL_REGX, var)
        return len(r.group()) > 0 if r else False

    @staticmethod
    def is_mobile(var: str):
        """
            判断是否是手机号
        :param var:
        :return:
        """
        if Utils.is_empty(var):
            return False
        r = re.search(Utils.MOBILE_REGX, var)
        return len(r.group()) > 0 if r else False


class TimeUtils(object):
    """
        时间 工具
    """
    DATETIME_FORMAT_DEFAULT = "%Y-%m-%d %H:%M:%S"
    DATETIME_FORMAT_MILLISECOND = "%Y-%m-%d %H:%M:%S.%f"
    DATE_FORMAT_DEFAULT = "%Y-%m-%d"

    @staticmethod
    def is_out_day(var: datetime, year=0, month=0, day=0, hour=0, minute=0, second=0):
        """
            判断时间是否超过了制定日期范围
        :param var: 制定时间
        :param year: 年限
        :param month: 月数
        :param day: 天数
        :param hour: 小时数
        :param minute: 分钟数
        :param second: 秒数
        :return:
        """
        now = datetime.now()
        #  当前时间比 参数大的时候，判断下限
        if now.__gt__(var):
            limit_day = datetime(year=now.year - year if year > 0 else now.year,
                                 month=now.month - month if month > 0 else now.month,
                                 day=now.day - day if day > 0 else now.day,
                                 hour=now.hour - hour if hour > 0 else now.hour,
                                 minute=now.minute - minute if minute > 0 else now.minute,
                                 second=now.second - second if second > 0 else now.second)
            return limit_day.__gt__(var)
        else:
            # 当前时间比参数小的时候 ，判断上限
            limit_day = datetime(year=now.year + year if year > 0 else now.year,
                                 month=now.month + month if month > 0 else now.month,
                                 day=now.day + day if day > 0 else now.day,
                                 hour=now.hour + hour if hour > 0 else now.hour,
                                 minute=now.minute + minute if minute > 0 else now.minute,
                                 second=now.second + second if second > 0 else now.second)
            return limit_day.__lt__(var)

    @staticmethod
    def get_now_str(_format: str):
        """
            获取当前时间的格式化字符串
        :param _format:
        :return:
        """
        return datetime.now().strftime(_format)

    @staticmethod
    def end_day_surplus_second(var: datetime = None):
        """
            获取指定时间当天的剩余时间
        :param var:  None able ，default now
        :return:
        """
        if var is None:
            var = datetime.now()
        end_of_day = TimeUtils.get_end_of_day(var)
        return TimeUtils.get_datetime_diff(var, end_of_day)

    @staticmethod
    def is_in_month(var1: datetime, var2: datetime):
        """
            判断同一个月 跨月按30天算 ，不跨越按自然月算
        :param var1:
        :param var2:
        :return:
        """
        in_same_month = TimeUtils.is_in_same_month(var1, var2)
        if in_same_month:
            return True
        diff = TimeUtils.get_datetime_diff_day(var1, var2)
        return abs(diff) <= 30

    @staticmethod
    def is_in_same_month(var1: datetime, var2: datetime):
        """
            判断同一个月 月份相同
        :param var1:
        :param var2:
        :return:
        """
        return var1.year == var2.year and var1.month == var2.month

    @staticmethod
    def is_in_day(var1: datetime, var2: datetime):
        """
            是否是一天内 ，判断的是秒数
        :param var1:
        :param var2:
        :return:
        """
        diff_seconds = TimeUtils.get_datetime_diff(var1, var2)
        return abs(diff_seconds) <= 86400

    @staticmethod
    def get_datetime_diff(var1: datetime, var2: datetime):
        """
            获取两个时间的 秒差
        :param var1:
        :param var2:
        :return:
        """
        diff = (var2 - var1).seconds
        day_diff = TimeUtils.get_datetime_diff_day(var1, var2)
        return day_diff * 3600 * 24 + diff

    @staticmethod
    def get_datetime_diff_day(var1: datetime, var2: datetime):
        """
            获取两个时间日期差
        :param var1:
        :param var2:
        :return:
        """
        return (var2.date() - var1.date()).days

    @staticmethod
    def get_end_of_week(var, _format: str = None) -> datetime:
        """
            获取一周结束时间
        :param var:
        :param _format:
        :return:
        """
        t = TimeUtils.get_begin_of_week(var, _format)
        if t is None:
            raise TimeConversionTypeError
        return TimeUtils.get_end_of_day(t + timedelta(days=6))

    @staticmethod
    def get_begin_of_week(var, _format: str = None) -> datetime:
        """
            获取一周开始时间
        :param var:
        :param _format:
        :return:
        """
        t = TimeUtils.get_begin_of_day(var, _format)
        if t is None:
            raise TimeConversionTypeError
        return t - timedelta(days=t.weekday())

    @staticmethod
    def get_end_of_month(var, _format: str = None) -> datetime:
        """
            获取月底时间
        :param var: 入参的类型 可以是 str ， datetime ， date ，  struct_time
        :param _format:
        :return:
        """
        t = TimeUtils.get_begin_of_month(var, _format)
        if t is None:
            raise TimeConversionTypeError
        t += timedelta(days=32)
        return TimeUtils.get_end_of_day(t - timedelta(days=t.day))

    @staticmethod
    def get_begin_of_month(var, _format: str = None) -> datetime:
        """
            获取月初时间
        :param var:入参的类型 可以是 str ， datetime ， date ，  struct_time
        :param _format:
        :return:
        """
        t = TimeUtils.get_begin_of_month(var, _format)
        if t is None:
            raise TimeConversionTypeError
        return TimeUtils.get_begin_of_day(datetime(year=t.year, month=t.month, day=1))

    @staticmethod
    def get_end_of_day(var, _format: str = None) -> datetime:
        """
            获取一天结束时间
        :param var:入参的类型 可以是 str ， datetime ， date ，  struct_time
        :param _format:
        :return:
        """
        t = TimeUtils.get_datetime(var, _format)
        if t is None:
            raise TimeConversionTypeError
        return t.replace(hour=23, minute=59, second=59)

    @staticmethod
    def get_begin_of_day(var, _format: str = None) -> datetime:
        """
            获取一天开始时间
        :param var:
        :param _format:
        :return:
        """
        t = TimeUtils.get_datetime(var, _format)
        if t is None:
            raise TimeConversionTypeError
        return t.replace(hour=0, minute=0, second=0, microsecond=0)

    @staticmethod
    def get_datetime(var, _format: str = None):
        """
            时间转成 datetime
        :param var:
        :param _format:
        :return:
        """
        if var is None:
            raise NullPointError
        if isinstance(var, str):
            if _format is None:
                raise TimeNoFormatError
            return datetime.strptime(var, _format)
        elif isinstance(var, datetime):
            return var
        elif isinstance(var, time.struct_time):
            return datetime(year=var.tm_year, month=var.tm_mon, day=var.tm_mday, hour=var.tm_hour, minute=var.tm_min,
                            second=var.tm_sec)
        elif isinstance(var, date):
            return datetime(year=var.year, month=var.year, day=var.day)

    @staticmethod
    def get_timestamp(var=None, _format: str = None) -> int:
        """
            获取时间戳
        :param var: 时间 如果为空 表示获取当前时间的时间戳
        :param _format:  时间格式 , 当 var 是 str 的时候，需要传入
        :return:
        """
        t = datetime.now() if var is None else TimeUtils.get_datetime(var, _format)
        if t is None:
            raise TimeConversionTypeError
        # if var is None or isinstance(var, datetime):
        #     if var is None:
        #         timestamp = int(datetime.now().timestamp() * 1000)
        #     else:
        #         timestamp = int(var.timestamp() * 1000)
        # elif isinstance(var, str):
        #     if is_empty(_format):
        #         raise TimeNoFormatError()
        #     # 如果是字符串类型的时间格式 ， 必须有时间格式
        #     time_array = time.strptime(var, _format)
        #     timestamp = int(time.mktime(time_array))
        #     timestamp = timestamp * 1000
        # elif isinstance(var, date):
        #     timestamp = int(
        #         datetime(year=var.year, month=var.month, day=var.day, hour=0, minute=0, second=0).timestamp() * 1000)
        # else:
        #     raise TimeConversionTypeError
        return int(t.timestamp() * 1000)


import loguru

logger = loguru.logger
if __name__ == '__main__':
    try:
        a = {"a": "dadf", "bac": "xx", "a1": None}
        en = Entity(a)
        print(en.a1)
        v = TimeUtils.get_datetime(time.strptime("2022-03-26 03:00:00", TimeUtils.DATETIME_FORMAT_DEFAULT))
        v2 = TimeUtils.get_datetime(time.strptime("2023-03-29 11:24:50", TimeUtils.DATETIME_FORMAT_DEFAULT))
        logger.info(random.Random(TimeUtils.get_timestamp()).randrange(0, 999))
    except TimeNoFormatError as e:
        logger.exception(e)



class Error(Exception):
    def __init__(self, code, message):
        super().__init__(f"{message}[CODE:{code}]")


class TimeError(Error):
    """
        时间类的错误 code 10300 - 10399
    """
    pass


class TimeNoFormatError(TimeError):
    def __init__(self):
        """
            时间格式确实
        """
        super().__init__(code=10300, message="时间格式缺失")


class TimeConversionTypeError(TimeError):
    def __init__(self):
        """ 时间转换格式不正确"""
        super().__init__(code=10301, message="转换时间类型不是标准的时间类型")


class ConversionTypeError(Error):
    def __init__(self, message=None):
        super().__init__(code=10001, message=message and "类型错误:{}".format(message) or "目标类型错误")


class NullPointError(Error):
    def __init__(self):
        """ 空指针通用类型 """
        super().__init__(code=10000, message="空指针")


class ParamError(Error):
    def __init__(self):
        super().__init__(code=10002, message="入参不正确")


class PageError(Error):
    def __init__(self):
        super().__init__(code=10003, message="页码不能为空或小于0")


class SizeError(Error):
    def __init__(self):
        super().__init__(code=10004, message="分页展示数量不能小于0")


class OverSizeLimitError(Error):
    def __init__(self):
        super().__init__(code=10005, message="分页展示数量超过限制")


class VersionLowError(Error):
    def __init__(self, name, version):
        super().__init__(code=10006, message=f"当前{name}版本过低，请使用制定版本[{version}]")


class SQLFormatError(Error):
    def __init__(self, sql):
        super().__init__(code=10101, message=f"当前sql有误[{sql}]")


class SQLNotInsertError(Error):
    def __init__(self, sql):
        super().__init__(code=10102, message=f"当前sql不是INSERT语句[{sql}]")


class SQLNotUpdateError(Error):
    def __init__(self, sql):
        super().__init__(code=10103, message=f"当前sql不是UPDATE语句[{sql}]")


class SQLNotDeleteError(Error):
    def __init__(self, sql):
        super().__init__(code=10104, message=f"当前sql不是DELETE语句[{sql}]")


class SQLNotSelectError(Error):
    def __init__(self, sql):
        super().__init__(code=10105, message=f"当前sql不是DELETE语句[{sql}]")



import os

import loguru

TRACE = "TRACE"
INFO = "INFO"
DEBUG = "DEBUG"
ERROR = "ERROR"
WARNING = "WARNING"
fmt = "[%(thread)d]-[%(levelname)s]-[%(asctime)s}]-[%(process)d]-[%(name)s]-[%(funcName)s]%(lineno)s :%(message)s"


class Logger(object):
    LOGURU_FORMAT = loguru._defaults.env(
        "LOGURU_FORMAT",
        str,
        "[{process}]-"
        "[<level>{level: <8}</level>]-"
        "[<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green>]-"
        "[<cyan>{thread}</cyan>]-"
        "[<cyan>{file}</cyan>]-"
        "[<cyan>{function}</cyan>]<cyan>{line}</cyan> :"
        "<level>{message}</level>",
    )
    LOGURU_ERROR_FORMAT = loguru._defaults.env(
        "LOGURU_FORMAT",
        str,
        "[<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | "
        "<level>{level: <8}</level> | "
        "<cyan>{process} | {thread}</cyan> | "
        "<cyan>{file}</cyan>-><cyan>{function}</cyan>:<cyan>{line}</cyan> ]"
        "<level>{message}</level>",
    )

    @staticmethod
    def get_logger(file_name, project_name: str = None):
        """
            创建日志
        :param file_name:  保存日志文件名称 ，
        :param project_name:
        :return:
        """
        file_parent_path = "/var/log/duyansoft/py3log"
        if project_name:
            file_parent_path += ("/" + project_name)
        if not os.path.exists(file_parent_path):
            os.makedirs(file_parent_path, exist_ok=True)
        info_file_path = f"{file_parent_path}/{file_name}-{INFO}.log"
        error_file_path = f"{file_parent_path}/{file_name}-{ERROR}.log"
        return Logger.get_logger_init(info_file_path, error_file_path, project_name=project_name)

    @staticmethod
    def get_logger_init(info_file_path, error_file_path, _format=LOGURU_FORMAT, level=INFO, rotation="12:00",
                        retention="3 days",
                        compression="zip", enqueue=False, project_name: str = None):
        logger_obj = loguru.logger

        logger_obj.add(info_file_path, format=_format, level=level, enqueue=enqueue, rotation=rotation,
                       retention=retention, compression=compression)
        # 错误日志默认保存三十天 错误日志格式和保存时间是无法配置的
        logger_obj.add(error_file_path, format=Logger.LOGURU_FORMAT, level=ERROR, enqueue=enqueue,
                       rotation=rotation, retention="30 days", compression=compression)
        return logger_obj




import time
from enum import Enum

import selenium

from selenium import webdriver
from selenium.webdriver.common.by import By


class CatAction(Enum):
    CLICK = 1
    INPUT = 2
    GET_ELEMENT = 3
    GET_ELEMENTS = 4
    CLEAR = 5
    SUBMIT = 6
    GET_VALUE = 7


class ItemInfo(object):
    def __init__(self, send_value=None, attribute_name=None):
        self.send_value = send_value
        self.attribute_name = attribute_name


class Cat(object):
    def __init__(self, by: By, name: str, action: CatAction, item_info: ItemInfo = None):
        self.by = by
        self.name = name
        self.action = action
        self.item_info = item_info

    def do(self, driver):
        if CatAction.GET_ELEMENTS == self.action:
            return driver.find_elements(self.by, self.name)
        elif CatAction.GET_ELEMENT == self.action:
            return driver.find_element(self.by, self.name)
        elif CatAction.CLICK == self.action:
            driver.find_element(self.by, self.name).click()
            return None
        elif CatAction.INPUT == self.action:
            driver.find_element(self.by, self.name).send_keys(self.item_info.send_value)
            return None
        elif CatAction.CLEAR == self.action:
            driver.find_element(self.by, self.name).clear()
        elif CatAction.SUBMIT == self.action:
            driver.find_element(self.by, self.name).submit()
        elif CatAction.GET_VALUE == self.action:
            if self.item_info.attribute_name == "text":
                return driver.text
            return driver.get_attribute(self.item_info.attribute_name)
        return None


class WebDriver(object):

    def __init__(self, executable_path):
        self.executable_path = executable_path
        self.driver = webdriver.Chrome(executable_path)
        self.driver.maximize_window()
        a = self.driver.find_element().text

if __name__ == '__main__':
    driver = webdriver.Chrome(executable_path)



# from setuptools import setup, find_packages
#
# setup(
#     name="DuyanUtils",
#     version="1.1",
#     packages=find_packages("src"),
#     package_dir={"": "src"}
# )
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="DuyanUtils",
    version="1.0.3",
    author="Jianguo.Wang",
    author_email="chunyang.wang@duyansoft.com",
    description="常用工具 轻量化处理",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    # project_urls={
    #     "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    # },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)
